searchData={"items":[{"type":"module","title":"CTE","doc":"The Closure Table for Elixir strategy, CTE for short,\nis a simple and elegant way of storing and working with\nhierarchies. It involves storing all paths through a tree,\nnot just those with a direct parent-child relationship. You\nmay want to chose this model, over the [Nested Sets model](https://en.wikipedia.org/wiki/Nested_set_model),\nshould you need referential integrity and to assign nodes to multiple trees.\n\nWith CTE you can navigate through hierarchies using a\nsimple [API](CTE.Adapter.html#functions), such as:\nfinding the ascendants and descendants of a node, inserting\nand deleting nodes, moving entire sub-trees or print them\nas a digraph (.dot) file.\n\n#","ref":"CTE.html"},{"type":"module","title":"Quick example. - CTE","doc":"For this example we're using the in-[Memory Adapter](CTE.Adapter.Memory.html#content).\nThis `Adapter` is useful for prototyping or with data structures\nthat can easily fit in memory; their persistence being taken care\nof by other components. For more involved use cases, CTE integrates\nwith Ecto using a simple API.\n\nWhen used from a module, the CTE expects the: `:otp_app` and\n`:adapter` attributes, to be defined. The `:otp_app` should point\nto an OTP application that might provide additional configuration.\nEqually so are the: `:nodes` and the `:paths` attributes. The `:nodes`\nattribute, in the case of the [Memory Adapter](CTE.Adapter.Memory.html#content),\nis a Map defining your nodes while the: `:paths` attribute, is a list\ncontaining the tree path between the nodes - a list of lists. For example:\n\n    defmodule CTM do\n      use CTE,\n        otp_app: :ct_empty,\n        adapter: CTE.Adapter.Memory,\n        nodes: %{\n          1 => %{id: 1, author: \"Olie\", comment: \"Is Closure Table better than the Nested Sets?\"},\n          2 => %{id: 2, author: \"Rolie\", comment: \"It depends. Do you need referential integrity?\"},\n          3 => %{id: 3, author: \"Olie\", comment: \"Yeah.\"}\n        },\n        paths: [[1, 1, 0], [1, 2, 1], [1, 3, 2], [2, 2, 0], [2, 3, 1], [3, 3, 0]]\n    end\n\n\nWhen using the `CTE.Adapter.Ecto`, the: `:nodes` attribute,\nwill be a Schema i.e. `Post`, `TreePath`, etc! In our initial\nimplementation, the nodes definitions must have at least the\n`:id`, as one of their properties. This caveat will be lifted\nin a later implementation.\n\nAdd the `CTM` module to your main supervision tree:\n\n    defmodule CTM.Application do\n      @moduledoc false\n\n      use Application\n\n      def start(_type, _args) do\n        opts = [strategy: :one_for_one, name: CTM.Supervisor]\n\n        Supervisor.start_link([CTM], opts)\n      end\n    end\n\nUsing `iex -S mix`, for quickly experimenting with the CTE API:\n\n- find the descendants of comment #1\n\n    ```elixir\n    iex» CTM.descendants(1)\n    {:ok, [3, 2]}\n    ```\n\n- find the ancestors\n\n    ```elixir\n    iex» CTM.ancestors(2)\n    {:ok, [1]}\n\n    iex» CTM.ancestors(3)\n    {:ok, [1]}\n    ```\n- find the ancestors, with information about the node:\n\n    ```elixir\n    iex» CTM.ancestors(2, nodes: true)\n    {:ok,\n    [\n      %{\n        author: \"Olie\",\n        comment: \"Is Closure Table better than the Nested Sets?\",\n        id: 1\n      }\n    ]}\n    ```\n- move leafs/subtrees around. From being a child of comment #1, to becoming a\nchild of comment #2, in the following example:\n\n    ```elixir\n    iex» CTM.move(3, 2)\n    :ok\n    iex» CTM.descendants(2)\n    {:ok, [3]}\n    ```\n\nPlease check the docs, the tests, and the examples folder, for more details.","ref":"CTE.html#module-quick-example"},{"type":"type","title":"CTE.config/0","doc":"","ref":"CTE.html#t:config/0"},{"type":"type","title":"CTE.name/0","doc":"","ref":"CTE.html#t:name/0"},{"type":"type","title":"CTE.nodes/0","doc":"","ref":"CTE.html#t:nodes/0"},{"type":"type","title":"CTE.paths/0","doc":"","ref":"CTE.html#t:paths/0"},{"type":"type","title":"CTE.repo/0","doc":"","ref":"CTE.html#t:repo/0"},{"type":"type","title":"CTE.t/0","doc":"","ref":"CTE.html#t:t/0"},{"type":"type","title":"CTE.table/0","doc":"","ref":"CTE.html#t:table/0"},{"type":"behaviour","title":"CTE.Adapter","doc":"Specification of the Closure Table implementation.\n\nMost of the functions implementing the `CTE.Adapter` behavior, will accept the following options:\n\n- `:limit`, to limit the total number of nodes returned, when finding the ancestors or the descendants for nodes\n- `:itself`, accepting a boolean value. When `true`, the node used for finding its neighbors are returned as part of the results. Default: true\n- `:nodes`, accepting a boolean value. When `true`, the results are containing additional information about the nodes. Default: false","ref":"CTE.Adapter.html"},{"type":"callback","title":"CTE.Adapter.ancestors/3","doc":"Retrieve the ancestors of a node","ref":"CTE.Adapter.html#c:ancestors/3"},{"type":"callback","title":"CTE.Adapter.delete/2","doc":"Delete a leaf or a subtree.\nhĕdzˈŭpˈ: read the docs of the implementation, in case the implementation has specific side effects","ref":"CTE.Adapter.html#c:delete/2"},{"type":"callback","title":"CTE.Adapter.delete/3","doc":"","ref":"CTE.Adapter.html#c:delete/3"},{"type":"callback","title":"CTE.Adapter.descendants/3","doc":"Retrieve the descendants of a node","ref":"CTE.Adapter.html#c:descendants/3"},{"type":"callback","title":"CTE.Adapter.insert/4","doc":"Insert a node under an existing ancestor","ref":"CTE.Adapter.html#c:insert/4"},{"type":"callback","title":"CTE.Adapter.move/4","doc":"Move a subtree from one location to another.\n\nFirst, the subtree and its descendants are disconnected from its ancestors. And second, the subtree is inserted under the new parent (ancestor) and the subtree, including its descendants, is declared as descendants of all the new ancestors.","ref":"CTE.Adapter.html#c:move/4"},{"type":"callback","title":"CTE.Adapter.tree/3","doc":"Calculate and return a \"tree\" structure containing the paths and the nodes under the given leaf/node","ref":"CTE.Adapter.html#c:tree/3"},{"type":"type","title":"CTE.Adapter.options/0","doc":"","ref":"CTE.Adapter.html#t:options/0"},{"type":"type","title":"CTE.Adapter.t/0","doc":"","ref":"CTE.Adapter.html#t:t/0"},{"type":"module","title":"CTE.Adapter.Ecto","doc":"A CTE Adapter implementation using an existing Ecto Repo for persisting the models.\n\nThe current implementation is depending on Ecto ~> 3.1; using [Ecto.SubQuery](https://hexdocs.pm/ecto/Ecto.SubQuery.html)!\n\nFor this implementation to work you'll have to provide two tables, and the name of the Repo used by your application:\n\n1. a table name containing the nodes. Having the `id`, as a the primary key\n2. a table name where the tree paths will be stores.\n3. the name of the Ecto.Repo, defined by your app\n\nIn a future version we will provide you with a convenient migration template to help you starting, but for now you must supply these tables.\n\nFor example, given you have the following Schemas for comments:\n\n    defmodule CT.Comment do\n      use Ecto.Schema\n      import Ecto.Changeset\n\n      @timestamps_opts [type: :utc_datetime]\n\n      schema \"comments\" do\n        field :text, :string\n        belongs_to :author, CT.Author\n\n        timestamps()\n      end\n    end\n\nand a table used for storing the parent-child relationships\n\n    defmodule CT.TreePath do\n      use Ecto.Schema\n      import Ecto.Changeset\n      alias CT.Comment\n\n      @primary_key false\n\n      schema \"tree_paths\" do\n        belongs_to :parent_comment, Comment, foreign_key: :ancestor\n        belongs_to :comment, Comment, foreign_key: :descendant\n        field :depth, :integer, default: 0\n      end\n    end\n\nwe can define the following module:\n\n    defmodule CT.MyCTE do\n      use CTE,\n      otp_app: :cte,\n      adapter: CTE.Adapter.Ecto,\n      repo: CT.Repo,\n      nodes: CT.Comment,\n      paths: CT.TreePath\n    end\n\n\nWe add our CTE Repo to the app's main supervision tree, like this:\n\n    defmodule CT.Application do\n      use Application\n\n      def start(_type, _args) do\n        children = [\n          CT.Repo,\n          CT.MyCTE\n        ]\n\n        opts = [strategy: :one_for_one, name: CT.Supervisor]\n        Supervisor.start_link(children, opts)\n      end\n    end\n\nrestart out app and then using IEx, we can start experimenting. Examples:\n\n    iex» CT.MyCTE.ancestors(9)\n    {:ok, [1, 4, 6]}\n\n    iex» CT.MyCTE.tree(6)\n    {:ok,\n    %{\n    nodes: %{\n      6 => %CT.Comment{\n        __meta__: #Ecto.Schema.Metadata ,\n        author: #Ecto.Association.NotLoaded ,\n        author_id: 2,\n        id: 6,\n        inserted_at: ~U[2019-07-21 01:10:35Z],\n        text: \"Everything is easier, than with the Nested Sets.\",\n        updated_at: ~U[2019-07-21 01:10:35Z]\n      },\n      8 => %CT.Comment{\n        __meta__: #Ecto.Schema.Metadata ,\n        author: #Ecto.Association.NotLoaded ,\n        author_id: 1,\n        id: 8,\n        inserted_at: ~U[2019-07-21 01:10:35Z],\n        text: \"I’m sold! And I’ll use its Elixir implementation!  ,\n        author: #Ecto.Association.NotLoaded ,\n        author_id: 3,\n        id: 9,\n        inserted_at: ~U[2019-07-21 01:10:35Z],\n        text: \"w⦿‿⦿t!\",\n        updated_at: ~U[2019-07-21 01:10:35Z]\n      }\n    },\n    paths: [\n            [6, 6, 0],\n            [6, 8, 1],\n            [8, 8, 0],\n            [6, 9, 1],\n            [9, 9, 0]\n          ]\n    }}\n\nHave fun!\n\n\nMost of the functions implementing the `CTE.Adapter` behavior, will accept the following options:\n\n- `:limit`, to limit the total number of nodes returned, when finding the ancestors or the descendants for nodes\n- `:itself`, accepting a boolean value. When `true`, the node used for finding its neighbors are returned as part of the results. Default: true\n- `:nodes`, accepting a boolean value. When `true`, the results are containing additional information about the nodes. Default: false","ref":"CTE.Adapter.Ecto.html"},{"type":"function","title":"CTE.Adapter.Ecto.ancestors/3","doc":"Retrieve the ancestors of a node","ref":"CTE.Adapter.Ecto.html#ancestors/3"},{"type":"function","title":"CTE.Adapter.Ecto.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"CTE.Adapter.Ecto.html#child_spec/1"},{"type":"function","title":"CTE.Adapter.Ecto.delete/3","doc":"Delete a leaf or a subtree.\n\nTo delete a leaf node set the limit option to: 1, and in this particular case\nall the nodes that reference the leaf will be assigned to the leaf's immediate ancestor\n\nIf limit is 0, then the leaf and its descendants will be deleted","ref":"CTE.Adapter.Ecto.html#delete/3"},{"type":"function","title":"CTE.Adapter.Ecto.descendants/3","doc":"Retrieve the descendants of a node","ref":"CTE.Adapter.Ecto.html#descendants/3"},{"type":"function","title":"CTE.Adapter.Ecto.init/1","doc":"Initializes the adapter supervision tree by returning the children and adapter metadata.","ref":"CTE.Adapter.Ecto.html#init/1"},{"type":"function","title":"CTE.Adapter.Ecto.insert/4","doc":"Insert a node under an existing ancestor","ref":"CTE.Adapter.Ecto.html#insert/4"},{"type":"function","title":"CTE.Adapter.Ecto.move/4","doc":"Move a subtree from one location to another.\n\nFirst, the subtree and its descendants are disconnected from its ancestors. And second, the subtree is inserted under the new parent (ancestor) and the subtree, including its descendants, is declared as descendants of all the new ancestors.","ref":"CTE.Adapter.Ecto.html#move/4"},{"type":"function","title":"CTE.Adapter.Ecto.start_link/1","doc":"start the Adapter server","ref":"CTE.Adapter.Ecto.html#start_link/1"},{"type":"function","title":"CTE.Adapter.Ecto.tree/3","doc":"Calculate and return a \"tree\" structure containing the paths and the nodes under the given leaf/node","ref":"CTE.Adapter.Ecto.html#tree/3"},{"type":"module","title":"CTE.Adapter.Memory","doc":"Basic implementation of the CTE, using the memory for persisting the models. Adapter provided as a convenient way of using CTE in tests or during the development","ref":"CTE.Adapter.Memory.html"},{"type":"function","title":"CTE.Adapter.Memory.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"CTE.Adapter.Memory.html#child_spec/1"},{"type":"function","title":"CTE.Adapter.Memory.delete/3","doc":"Delete a leaf or a subtree.\n\nTo delete a leaf node set the limit option to: 1, and in this particular case\nall the nodes that reference the leaf will be assigned to the leaf's immediate ancestor\n\nIf limit is 0, then the leaf and its descendants will be deleted","ref":"CTE.Adapter.Memory.html#delete/3"},{"type":"function","title":"CTE.Adapter.Memory.init/1","doc":"Initializes the adapter supervision tree by returning the children and adapter metadata.","ref":"CTE.Adapter.Memory.html#init/1"},{"type":"function","title":"CTE.Adapter.Memory.start_link/1","doc":"start the Adapter server","ref":"CTE.Adapter.Memory.html#start_link/1"},{"type":"module","title":"CTE.Utils","doc":"Basic utilities for helping developing functionality about the `CTE` data structures","ref":"CTE.Utils.html"},{"type":"function","title":"CTE.Utils.print_dot/2","doc":"render a path hierarchy as a .dot string that you could use for drawing your results, using graphviz.  w⦿‿⦿t!\n\nUpon receiving something like this:\n\n    %{\n      nodes: %{\n        6 => %{\n          author: \"Rolie\",\n          comment: \"Everything is easier, than with the Nested Sets.\",\n          id: 6\n        },\n        8 => %{\n          author: \"Olie\",\n          comment: \"I’m sold! And I’ll use its Elixir implementation!  .dot -o  .png`","ref":"CTE.Utils.html#print_dot/2"},{"type":"function","title":"CTE.Utils.print_tree/3","doc":"print the tree at the console, using a custom function for selecting the info to be displayed\n\nThe print_tree/3 function receives the tree structure returned by the CTE, the id of an existing node we\nwant to start printing the tree with, followed by the options.","ref":"CTE.Utils.html#print_tree/3"},{"type":"function","title":"Options - CTE.Utils.print_tree/3","doc":"* `:callback` - a function that it is invoked for every node in the tree. Has two parameters:\n\n    * id - the id of the node we render, ao a node specific unique identifier i.e. node name, etc.\n    * nodes - the nodes received the tree structure, a map %{node_id => node}...any()\n\n    This function must return a tuple with two elements. First element is the name of the node we render,\n    the second one being any optional info you want to add.\n\n  * `:raw` - return a list of tuples if true. Each tuple will contain the depth of\n    the text returned from the callback. Useful for custom formatting the output of the print.\n\nExample, using the default options:\n\niex» {:ok, tree} = CTT.tree(1)\niex» CTE.Utils.print_tree(tree,1, callback: &({&2[&1].author   \":\", &2[&1].comment}))\n\nOlie: Is Closure Table better than the Nested Sets?\n└── Rolie: It depends. Do you need referential integrity?\n └── Olie: Yeah.","ref":"CTE.Utils.html#print_tree/3-options"},{"type":"function","title":"CTE.Utils.tree_to_map/3","doc":"converts the given tree into a nested map\n\nThis function receives a tree structure returned by the CTE and\nthe id of an existing node we want to start from. It returns\na nested map representation of the tree.","ref":"CTE.Utils.html#tree_to_map/3"},{"type":"function","title":"Options - CTE.Utils.tree_to_map/3","doc":"* `:callback` - a function of arity one that accepts the current\n    node as input and outputs a formatted node. Defaults to `& &1`.\n\nExample:\n\niex» {:ok, tree} = CTT.tree(1)\niex» CTE.Utils.tree_to_map(tree, 6, callback: &Map.take(&1, [:text]))\n\n%{\n  \"children\" => [\n    %{\n      \"children\" => [],\n      \"id\" => 8,\n      \"node\" => %{text: \"I’m sold! And I’ll use its Elixir implementation! <3\"}\n    },\n    %{\"children\" => [], \"id\" => 9, \"node\" => %{text: \"w⦿‿⦿t!\"}}\n  ],\n  \"id\" => 6,\n  \"node\" => %{text: \"Everything is easier, than with the Nested Sets.\"}\n}","ref":"CTE.Utils.html#tree_to_map/3-options"},{"type":"extras","title":"Closure Table","doc":"# Closure Table\n\n[![Hex.pm](https://img.shields.io/hexpm/dt/closure_table.svg?maxAge=2592000)](https://hex.pm/packages/closure_table)\n[![Hexdocs.pm](https://img.shields.io/badge/api-hexdocs-brightgreen.svg)](https://hexdocs.pm/closure_table)\n\n> this library provides two adapters: an in-memory one, and one for using the closure-table solution with Ecto; for your testing and development convenience.\n\nThe Closure Table solution is a simple and elegant way of storing hierarchies. It involves storing all paths through a tree, not just those with a direct parent-child relationship. You may want to chose this model, over the [Nested Sets model](https://en.wikipedia.org/wiki/Nested_set_model), should you need referential integrity and to assign nodes to multiple trees.\n\nThroughout the various examples and tests, we will refer to the hierarchies depicted below, where we're modeling a hypothetical forum-like discussion between [Rolie, Olie and Polie](https://www.youtube.com/watch?v=LTkmaE_QWMQ), and their debate around the usefulness of this implementation :)\n\n![Closure Table](assets/closure_table.png)","ref":"readme.html"},{"type":"extras","title":"Quick start - Closure Table","doc":"To start, you can simply use one `Adapter` from the ones provided, same way you'd use the Ecto's own Repo:\n\n```elixir\ndefmodule CTT do\n  use CTE,\n    otp_app: :cte,\n    adapter: CTE.Adapter.Memory,\n    nodes: %{\n      1 => %{id: 1, author: \"Olie\", comment: \"Is Closure Table better than the Nested Sets?\"},\n      2 => %{id: 2, author: \"Rolie\", comment: \"It depends. Do you need referential integrity?\"},\n      3 => %{id: 3, author: \"Olie\", comment: \"Yeah.\"}\n    },\n    paths: [[1, 1, 0], [1, 2, 1], [1, 3, 2], [2, 2, 0], [2, 3, 1], [3, 3, 0]]\nend\n```\n\nWith the configuration above, the `:nodes` attribute is a map containing the comments our interlocutors made; these are \"nodes\", in CTE's parlance. When using the `CTE.Adapter.Ecto` implementation, the `:nodes` attribute will be a Schema (or a table name! In our initial implementation, the nodes definitions must have at least the `:id`, as one of their properties. This caveat will be lifted in a later implementation. The `:paths` attribute represents the parent-child relationship between the comments.\n\nAdd the `CTM` module to your main supervision tree:\n\n```elixir\ndefmodule CTM.Application do\n  @moduledoc false\n\n  use Application\n\n  def start(_type, _args) do\n    opts = [strategy: :one_for_one, name: CTM.Supervisor]\n\n    Supervisor.start_link([CTM], opts)\n  end\nend\n```\n\nAnd then using `iex -S mix`, for quickly experimenting with the CTE API, let's find the descendants of comment #1:\n\n```elixir\niex» CTM.descendants(1)\n{:ok, [3, 2]}\niex> {:ok, tree} = CTT.tree(1)\n...\niex> CTE.Utils.print_tree(tree, 1)\n...\niex» CTE.Utils.print_tree(tree,1, callback: &({&2[&1].author   \":\", &2[&1].comment}))\n\nIs Closure Table better than the Nested Sets?\n└── It depends. Do you need referential integrity?\n   └── Yeah.\n\n```\n\nPlease check the docs for more details and return from more updates!\n\nOh and there is a simple utility for helping you drawing your paths, using graphviz! From Rolie's comments, excerpt:\n\n![dot](assets/dot.dot.dot.png)\n\nMaybe useful?! If yes, then we'll let you find this function by yourself ;)\n\nhint: _check the tests <3_","ref":"readme.html#quick-start"},{"type":"extras","title":"Installation - Closure Table","doc":"If [available in Hex](https://hex.pm/packages/closure_table), the package can be installed\nby adding `closure_table` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:closure_table, \"~> 1.1\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Contributing - Closure Table","doc":"- [Fork this project](https://github.com/florinpatrascu/closure_table/fork)\n- Create your feature branch (git checkout -b my-new-feature)\n- Setup database and test (`mix test`)\n- Commit your changes (`git commit -am 'Add some feature'`)\n- Push to the branch (`git push origin my-new-feature`)\n- Create new Pull Request","ref":"readme.html#contributing"},{"type":"extras","title":"License - Closure Table","doc":"```txt\nCopyright 2023 Florin T.PATRASCU & the Contributors\n\nLicensed under the Apache License, Version 2.0 (the \"License\");\nyou may not use this file except in compliance with the License.\nYou may obtain a copy of the License at\n\n  http://www.apache.org/licenses/LICENSE-2.0\n\nUnless required by applicable law or agreed to in writing, software\ndistributed under the License is distributed on an \"AS IS\" BASIS,\nWITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\nSee the License for the specific language governing permissions and\nlimitations under the License.\n```","ref":"readme.html#license"},{"type":"extras","title":"CHANGELOG","doc":"# CHANGELOG","ref":"changelog.html"},{"type":"extras","title":"1.1.5 - CHANGELOG","doc":"- dependencies update","ref":"changelog.html#1-1-5"},{"type":"extras","title":"1.1.4 - CHANGELOG","doc":"- code cleanup","ref":"changelog.html#1-1-4"},{"type":"extras","title":"1.1.3 - CHANGELOG","doc":"- improved CT tree structure map representation. The node's children are a list now, to preserve their original order - suitable for rendering the tree in UI, etc.","ref":"changelog.html#1-1-3"},{"type":"extras","title":"1.1.2 - CHANGELOG","doc":"- convert a CT tree structure to a nested map representation of the tree. This can be used for exporting this data as json, and so on. Thanks @greg-rychlewski","ref":"changelog.html#1-1-2"},{"type":"extras","title":"1.1.1 - CHANGELOG","doc":"- requiring elixir 1.11 or newer\n- CTE.Supervisor code cleanup, accepting a name parameter now\n- fix the Memory Adapter example\n- module documentation update; example was old and was missing the depth\n- dependencies upgrade, for main and for the Ecto simple demo app","ref":"changelog.html#1-1-1"},{"type":"extras","title":"1.0.11 - CHANGELOG","doc":"Minor changes:\n\n- Dependencies updated to latest versions. New projects should no longer have dependency conflicts when installing closure_table\n- Refactored Memory adapter's `delete` function (credo recommendation)\n- Updated documentation to reflect that the options passed to the Ecto adapter's `move/3` are currently ignored\n- Updated tests to remove some warnings","ref":"changelog.html#1-0-11"},{"type":"extras","title":"1.0.10 - CHANGELOG","doc":"Major changes, and a couple of significant bug fixes.\n\n- stable version! w⦿‿⦿t!\n- deleting nodes using option: `limit: 1`, behave as expected now across adapters. To delete a leaf node set the limit option to: 1, and in this particular case all the nodes that reference the leaf will be assigned to the leaf's immediate ancestor. When using `limit: 0`, the leaf and all its descendants will be deleted!\n- added tests for ASCII printing for both adapter; memory and Ecto\n- the ancestors and the descendants are returned now using the proper order (based on their `:depth` value)\n- improved tests for those cases where the outcome is complex or confusing\n- updated dependencies\n\nWarning: please use the move function with care! I believe it will require some refactoring and you'll get an eternal place in the CT's Hall of Fame if you'd like contributing to improving it!","ref":"changelog.html#1-0-10"},{"type":"extras","title":"1.0.0 (rc) - CHANGELOG","doc":"This is a new version introducing breaking changes. Please continue to use 0.2.1 for production, until we hit 1.2 unless you know what you're doing =)\n\n- the CT Ecto demo setup is much friendlier now, and it is also seeding the new database with demo data\n- the labels support functions now\n- the paths returned will always have the depth, as the 3rd element (breaking chance)\n- support for pretty printing trees. Example:\n\n```elixir\n  {:ok, tree} = CTT.tree(1)\n  CTE.Utils.print_tree(tree,1, callback: &({&2[&1].author   \":\", &2[&1].comment}))\n\n  Olie: Is Closure Table better than the Nested Sets?\n  └── Rolie: It depends. Do you need referential integrity?\n   └── Olie: Yeah.\n```","ref":"changelog.html#1-0-0-rc"},{"type":"extras","title":"0.2.1 - CHANGELOG","doc":"- support for immediate parent or child query #1; introducing the requirements for an extra field, in the tree_paths support table: `depth`","ref":"changelog.html#0-2-1"},{"type":"extras","title":"0.1.6 - CHANGELOG","doc":"- better tests covering the Ecto Adapter example","ref":"changelog.html#0-1-6"},{"type":"extras","title":"0.1.4 - CHANGELOG","doc":"- mix config fix.","ref":"changelog.html#0-1-4"},{"type":"extras","title":"0.1.3 - CHANGELOG","doc":"- fixing the examples using the CTE Adapter for Ecto - WIP","ref":"changelog.html#0-1-3"},{"type":"extras","title":"0.1.1 - CHANGELOG","doc":"- minor text adjustments\n- removing redundant content","ref":"changelog.html#0-1-1"},{"type":"extras","title":"0.1.0 - CHANGELOG","doc":"initial version","ref":"changelog.html#0-1-0"}],"content_type":"text/markdown"}